@@ -28,7 +28,7 @@
 #include <float.h>
 
 static char sccsid[] UNUSED = "@(#)options.c	3.5	09/25/04" ;
-static char svnid[] UNUSED = "$Id: dy_options.c 269 2009-04-02 05:38:19Z lou $" ;
+static char svnid[] UNUSED = "$Id: dy_options.c 307 2009-10-16 23:32:20Z lou $" ;
 
 /*
   Routines to acquire dylp's defaults and limits for options and tolerances.
@@ -722,7 +722,7 @@ static bool lpctl_finpurge (void)
       mkcref(zfinpurge_conminus1),mkcref(zpurge_ref),mkcref(zfinpurge_what) } ;
   static gdef(zfinpurge_def,
 	      sizeof(struct finpurge_struct),NULL,zfinpurge_alt) ;
-  static gref(zfinpurge,zfinpurge_def,bnfdebug,NULL,NULLP) ;
+  static gref(zfinpurge,zfinpurge_def,NULL,NULL,NULLP) ;
 
   dy_exposeOptDefaults(&opts_lb,&opts_dflt,&opts_ub) ;
 /*
