@@ -68,15 +68,144 @@
 
 #include "dylp.h"
 
-static char svnid[] UNUSED = "$Id: dy_rays.c 269 2009-04-02 05:38:19Z lou $" ;
+static char svnid[] UNUSED = "$Id: dy_rays.c 302 2009-10-08 00:29:17Z lou $" ;
+
+#if DYLP_PARANOIA > 0
 
 /* dy_tableau.c */
 
-#if DYLP_PARANOIA > 0
 extern bool dy_std_paranoia(const lpprob_struct *orig_lp,const char *rtnnme) ;
-#endif
 
 
+static bool check_dualRay (lpprob_struct *orig_lp, bool fullRay,
+			   int i_ray, int rayDir, double *ray, bool trueDuals)
+/*
+  This is a paranoid check routine used for debugging dy_dualRays. It
+  compares the ray passed as a parameter with the result of calculating
+  beta<i_orig>A on the original system. The routine works in the original
+  system frame of reference. It uses dy_betai to calculate beta<i_orig> and
+  dy_colStatus and dy_logStatus to retrieve the status for logical and
+  architectural variables. But we do cheat and use dy_tols.
+
+  Parameters:
+    orig_lp:	the lp problem structure
+    fullRay:	TRUE if ray includes components associated with nonbasic
+		architecturals, FALSE if it includes only components associated
+		with explicit constraints.
+    i_ray:	index of the row in the original constraint system used to
+		generate the ray
+    rayDir:	the direction of motion for the entering dual associated
+		with this ray
+    ray:	the ray to be checked
+    trueDuals:	true if the ray uses the true dual sign convention, false
+		to use dylp's min primal convention.
+
+  Returns: TRUE if the ray checks out, FALSE otherwise
+*/
+
+{ int m,n,i,j,errs ;
+  double *goldRay,*betai_ray,*y,*cbar ;
+  double rayk ;
+  bool retval ;
+
+  consys_struct *sys ;
+  flags *log_status,*arch_status ;
+
+  char *rtnnme = "check_dualRay" ;
+
+  sys = orig_lp->consys ;
+
+  if (dy_opts->print.rays >= 3)
+  { dyio_outfmt(dy_logchn,dy_gtxecho,"\n%s: checking ray from %s (%d) ... ",
+		rtnnme,consys_nme(sys,'c',i_ray,FALSE,NULL),i_ray) ; }
+/*
+  Acquire beta<i_ray>.
+*/
+  betai_ray = NULL ;
+  if (dy_betai(orig_lp,i_ray,&betai_ray) == FALSE)
+  { errmsg(952,rtnnme,sys->nme,"row",i_ray,"constraint",
+	   consys_nme(sys,'c',i_ray,FALSE,NULL),i_ray) ;
+    if (betai_ray != NULL) FREE(betai_ray) ;
+    return (FALSE) ; }
+/*
+  Acquire the status vectors for the logicals and architecturals, and the row
+  and column duals (a.k.a. duals and reduced costs).
+*/
+  log_status = NULL ;
+  dy_logStatus(orig_lp,&log_status) ;
+  arch_status = NULL ;
+  dy_colStatus(orig_lp,&arch_status) ;
+  y = NULL ;
+  dy_rowDuals(orig_lp,&y,TRUE) ;
+  cbar = NULL ;
+  dy_colDuals(orig_lp,&cbar,TRUE) ;
+/*
+  Allocate space for the golden ray. Always allocate space large enough for a
+  full ray. This is, after all, a paranoid routine and having the full ray
+  makes the code a bit simpler.
+*/
+  m = sys->concnt ;
+  n = sys->varcnt ;
+  goldRay = (double *) CALLOC((m+n+1),sizeof(double)) ;
+/*
+  Calculate the ray elements. For the logicals, the ray element is simply
+  the ith element of beta<i_ray>. For the architecturals, we need to do an
+  honest dot product. Keep in mind that inv(B)N is -(dualN)(inv(dualB)),
+  and that absorbs the minus sign that would otherwise appear here. Flip
+  the ray (-1) for NBUB variables.
+*/
+  for (i = 1 ; i <= m ; i++)
+  { if (flgon(log_status[i],vstatBASIC)) continue ;
+    goldRay[i] = rayDir*betai_ray[i] ; }
+  for (j = 1 ; j <= n ; j++)
+  { if (flgon(arch_status[j],vstatBASIC)) continue ;
+    rayk = rayDir*consys_dotcol(sys,j,betai_ray) ;
+    if (flgon(arch_status[j],vstatNBUB) && trueDuals == TRUE)
+      rayk = -rayk ;
+    goldRay[m+j] = rayk ; }
+/*
+  And compare
+*/
+  errs = 0 ;
+  for (i = 1 ; i <= m ; i++)
+  { if (dy_opts->print.rays >= 5)
+    { dyio_outfmt(dy_logchn,dy_gtxecho,"\n  y<%d> = %g, ray<%d> = %g",
+		  i,y[i],i,ray[i]) ; }
+    if (fabs(ray[i]-goldRay[i]) > dy_tols->cost)
+    { dyio_outfmt(dy_logchn,dy_gtxecho,
+		"\n  ERROR: ray<%d> = %g != %g = gold<%d>; error %g, tol %g.",
+		i,ray[i],goldRay[i],i,ray[i]-goldRay[i],dy_tols->cost) ;
+      errs++ ; } }
+  if (fullRay == TRUE)
+  { for (j = 1 ; j <= n ; j++)
+    { if (dy_opts->print.rays >= 5)
+      { dyio_outfmt(dy_logchn,dy_gtxecho,"\n  cbar<%d> = %g, ray<%d+%d> = %g",
+		    j,cbar[j],m,j,ray[m+j]) ; }
+      if (fabs(ray[m+j]-goldRay[m+j]) > dy_tols->cost)
+      { dyio_outfmt(dy_logchn,dy_gtxecho,
+	    "\n  ERROR: ray<%d+%d> = %g != %g = gold<%d+%d>; error %g, tol %g.",
+	    m,j,ray[m+j],goldRay[m+j],m,j,ray[m+j]-goldRay[m+j],dy_tols->cost) ;
+	errs++ ; } } }
+  if (errs == 0)
+  { if (dy_opts->print.rays >= 3)
+    { if (dy_opts->print.rays >= 4) dyio_outchr(dy_logchn,dy_gtxecho,'\n') ;
+      dyio_outfmt(dy_logchn,dy_gtxecho," ok.") ; }
+    retval = TRUE ; }
+  else
+  { if (dy_opts->print.rays >= 3)
+    { if (dy_opts->print.rays >= 4) dyio_outchr(dy_logchn,dy_gtxecho,'\n') ;
+      dyio_outfmt(dy_logchn,dy_gtxecho," %d errors.",errs) ; }
+    retval = FALSE ; }
+
+  if (log_status != NULL) FREE(log_status) ;
+  if (arch_status != NULL) FREE(arch_status) ;
+  if (betai_ray != NULL) FREE(betai_ray) ;
+  if (goldRay != NULL) FREE(goldRay) ;
+
+  return (retval) ; }
+
+#endif
+
 static bool testForPrimalRay (int j, int *p_dir, double **p_abarj)
 
 /*
@@ -112,10 +241,10 @@ static bool testForPrimalRay (int j, int *p_dir, double **p_abarj)
   Returns: TRUE if the evaluation completes without error, FALSE otherwise.
 */
 
-{ int k,m,kpos,dir ;
+{ int k = 0,m,kpos,dir ;
   flags statj,statk ;
   double *abarj ;
-  double abarkj,cbarj ;
+  double abarkj = 0.0,cbarj ;
   bool rayUp,rayDown ;
   
   const char *rtnnme = "testForPrimalRay" ;
@@ -235,9 +364,10 @@ static bool testForPrimalRay (int j, int *p_dir, double **p_abarj)
   toward lb<k>, and if abar<kj> < 0, we're moving x<k> toward ub<k>.
 */
   if (rayUp == TRUE)
-  { for (kpos = 1 ; kpos <= m && rayUp == TRUE ; kpos++)
-    { abarkj = abarj[kpos] ;
-      if (withintol(abarkj,0,dy_tols->zero)) continue ;
+  { for (kpos = 1 ; kpos <= m ; kpos++)
+    { setcleanzero(abarj[kpos],dy_tols->zero) ;
+      abarkj = abarj[kpos] ;
+      if (abarkj == 0.0) continue ;
       k = dy_basis[kpos] ;
       statk = dy_status[k] ;
       if (flgon(statk,vstatBFR)) continue ;
@@ -248,8 +378,7 @@ static bool testForPrimalRay (int j, int *p_dir, double **p_abarj)
 #   ifndef DYLP_NDEBUG
     if (dy_opts->print.rays >= 4)
     { if (rayUp == FALSE)
-      { kpos-- ;
-	dyio_outfmt(dy_logchn,dy_gtxecho,
+      { dyio_outfmt(dy_logchn,dy_gtxecho,
 		    "; basis pos'n %d: abar<%d,%d> = %g; %s (%d) ",
 		    kpos,k,j,abarkj,consys_nme(dy_sys,'v',k,FALSE,NULL),k) ;
 	if (abarkj < 0)
@@ -267,9 +396,10 @@ static bool testForPrimalRay (int j, int *p_dir, double **p_abarj)
   abar<kj> < 0 will move x<k> toward lb<k>.
 */
   if (rayDown == TRUE)
-  { for (kpos = 1 ; kpos <= m && rayDown == TRUE ; kpos++)
-    { abarkj = abarj[kpos] ;
-      if (withintol(abarkj,0,dy_tols->zero)) continue ;
+  { for (kpos = 1 ; kpos <= m ; kpos++)
+    { setcleanzero(abarj[kpos],dy_tols->zero) ;
+      abarkj = abarj[kpos] ;
+      if (abarkj == 0.0) continue ;
       k = dy_basis[kpos] ;
       statk = dy_status[k] ;
       if (flgon(statk,vstatBFR)) continue ;
@@ -280,8 +410,7 @@ static bool testForPrimalRay (int j, int *p_dir, double **p_abarj)
 #   ifndef DYLP_NDEBUG
     if (dy_opts->print.rays >= 4)
     { if (rayDown == FALSE)
-      { kpos-- ;
-	dyio_outfmt(dy_logchn,dy_gtxecho,
+      { dyio_outfmt(dy_logchn,dy_gtxecho,
 		    "; basis pos'n %d: abar<%d,%d> = %g; %s (%d) ",
 		    kpos,k,j,abarkj,consys_nme(dy_sys,'v',k,FALSE,NULL),k) ;
 	if (abarkj < 0)
@@ -349,6 +478,11 @@ bool dy_primalRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
   at the least). A call when the result of optimisation was anything other
   than unbounded will return zero rays.
 
+  Note that we don't have to worry about synthesizing ray elements for
+  inactive constraints. By definition, the basic variable for an inactive
+  constraint is the associated logical, and we don't report any ray
+  components for logical variables.
+
   Parameters:
     orig_lp:	the lp problem structure
     p_numRays:	(i) the maximum number of rays to return
@@ -374,7 +508,7 @@ bool dy_primalRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
   consys_struct *orig_sys ;
   bool scaled ;
   const double *rscale,*cscale ;
-  double Sj,dir ;
+  double invSj,dir ;
 
   int numCols, numRays,maxRays,rayDir,j_ray,j_orig_ray,i_orig_ray ;
   flags statj_ray ;
@@ -497,8 +631,8 @@ bool dy_primalRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
   testing x<B> = inv(B)b - abar<j>x<j>, so ray<j> = -abar<j>. Then, if x<j>
   is decreasing, that's another factor of -1 (encoded in rayDir). Finally, if
   the constraint in question is a >= constraint in the original system,
-  there's another factor of -1 folded into the row scaling (but we can't use
-  that directly; we want only the sign).
+  there's another factor of -1 folded into the direction of change of the
+  logical (which is really headed toward -inf in the original >= constraint).
 
   In terms of change of reference frame, we're moving from active system
   basis order to original system variable order. The translation is basis
@@ -506,7 +640,7 @@ bool dy_primalRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
   j_orig. Drop logicals, as they're not present in the original system.
 
   Last, but not least, x<j> is itself moving and must be part of the ray. Add
-  a coefficient of -1.0, adjusted for direction, directly in the original
+  a coefficient of 1.0, adjusted for direction, directly in the original
   reference frame.
 */
     ray = CALLOC((n_orig+1),sizeof(double)) ;
@@ -522,15 +656,15 @@ bool dy_primalRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
 
     if (scaled == TRUE)
     { if (logical == TRUE)
-      { Sj = rscale[i_orig_ray]*dir ; }
+      { invSj = rscale[i_orig_ray]*dir ; }
       else
-      { Sj = 1/cscale[j_orig_ray]*dir ; }
+      { invSj = 1/cscale[j_orig_ray]*dir ; }
       for (i = 1 ; i <= m ; i++)
       { if (sc_abarj[i] == 0) continue ;
 	j = dy_basis[i] ;
 	if (j <= dy_sys->concnt) continue ;
         j_orig = dy_actvars[j] ;
-	ray[j_orig] = Sj*sc_abarj[i]*cscale[j_orig] ;
+	ray[j_orig] = cscale[j_orig]*sc_abarj[i]*invSj ;
 	setcleanzero(ray[j_orig],dy_tols->zero) ; } }
     else
     { for (i = 1 ; i <= m ; i++)
@@ -544,7 +678,7 @@ bool dy_primalRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
     { FREE(sc_abarj) ;
       sc_abarj = NULL ; }
     if (logical == FALSE)
-    { ray[j_orig_ray] = -1.0*dir ; }
+    { ray[j_orig_ray] = 1.0*dir ; }
 
 #   ifndef DYLP_NDEBUG
     if (dy_opts->print.rays >= 5)
@@ -609,12 +743,29 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
     * There's no need to worry about degeneracy, numerical stability and all
       the other little considerations that go into a selecting a good pivot.
 
+  Remember that if we were actually using the dual matrices, we'd be checking
+  y<k> - abar<ik>delta<i>, where abar<i> is row i of (dualN)inv(dualB), to see
+  if we could drive some y<k> to zero by increasing y<i>. If all abar<ik> <= 0
+  and some abar<ik> < 0, we have a ray.
+
+  But we're using the primal matrices, y<k> is cbar<k>, abar<i> is row i of
+  inv(B)N = -(dualN)inv(dualB), and we have implicit bounds to cope with,
+  which means that y<k> could be increasing to zero (x<k> entering from UB) or
+  decreasing to zero (x<k> entering from LB) and y<i> could be increasing from
+  zero (x<i> increasing and leaving at LB) or decreasing from zero (x<i>
+  decreasing and leaving at UB).
+
+  So the test expression is
+    cbar<k> - (-abar<ik>)delta<i> = cbar<k> + abar<ik>delta<i>
+  and abar<i> is the potential ray. If we have
+    delta<i>*(-cbar<k>/abar<ik>) <= 0
+  for all k and strictly less than zero for at least one k, then we have a ray.
+
   NOTE: This routine works in the active system reference frame. The vector
 	returned is abar<i>. This is not quite a ray: it needs a coefficient
-	(-1.0) for y<i> and it must be negated to become the corresponding
-	ray. It makes more sense to do this in the client, once we know how
-	the ray is to be presented. See, for example, dy_dualRays, which
-	transforms the ray for use by the outside world.
+	(1.0) for y<i>. It makes more sense to do this in the client, once
+	we know how the ray is to be presented. See, for example,
+	dy_dualRays, which transforms the ray for use by the outside world.
 
   Parameters:
     i:		Index of the row to be evaluated
@@ -630,8 +781,8 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
 */
 
 { int bvi,j,k,m,n,dir ;
-  flags stati,statk ;
-  double abarik ;
+  flags stati,statk = 0 ;
+  double abarik = 0.0 ;
   bool rayUp,rayDown ;
 
   double *betai,*abari ;
@@ -681,12 +832,12 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
     default:
     { break ; } }
 /*
-  The action for no ray is simply to return TRUE. This is obscured by all the
+  The action for no ray is simply to return. This is obscured by all the
   printing in the debug version.
 */
 # ifdef DYLP_NDEBUG
   if (dir == 0)
-  { return (TRUE) ; }
+  { return ; }
 # else
   if (dir == 0)
   { if (dy_opts->print.rays >= 4)
@@ -704,12 +855,12 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
   We'll have to work for it. We can use dy_btran to retrieve beta<i> as
   e<i>inv(B). It's annoying, but allocate abari too, even though we may throw
   it away, 'cause otherwise we'll have no place to put coefficients while
-  we're working.
+  we're working. At least we don't need to initialise it.
 */
   m = dy_sys->concnt ;
   n = dy_sys->varcnt ;
   betai = CALLOC((m+1),sizeof(double)) ;
-  abari = CALLOC((n+1),sizeof(double)) ;
+  abari = MALLOC((n+1)*sizeof(double)) ;
   betai[i] = 1.0 ;
   dy_btran(betai) ;
 /*
@@ -717,35 +868,39 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
   Only one case will apply.
 
   Walk the columns (skipping basic variables) looking for coefficients that
-  will block motion.  We're looking at y<k> = cbar<k> - abar<ik>y<i> and
+  will block motion.  We're looking at y<k> = cbar<k> + abar<ik>delta<i> and
   asking whether we can drive y<k> to 0.  If we make it through all the
   columns without turning up a blocking coefficient, we have a ray.
 
   For an up ray (y<i> increasing), the blocking conditions are:
     * x<k> NBLB (hence cbar<k> >= 0) and abarik < 0
     * x<k> NBUB (hence cbar<k> <= 0) and abarik > 0
+    * x<k> NBFR (hence cbar<k> == 0) and abarik != 0
 
   If we happen to run across an NBFR variable with abarik != 0, we're
-  immediately done. Dual feasibility is possible only with cbar<k> = 0.
-  NBFX variables, on the other hand, can never block --- we can regard them as
-  NBLB or NBUB as needed.
+  immediately done. Dual feasibility is possible only with cbar<k> == 0.
+  NBFX variables, on the other hand, can never block --- we can regard them
+  as NBLB or NBUB as needed. SB variables should not occur, but treat them as
+  NBFR if seen.
 */
   if (rayUp == TRUE)
   { for (k = 1 ; k <= n ; k++)
-    { statk = getflg(dy_status[k],vstatSTATUS) ;
-      if (flgon(statk,vstatBASIC|vstatNBFX)) continue ;
+    { abari[k] = 0 ;
+      statk = getflg(dy_status[k],vstatSTATUS) ;
+      if (flgon(statk,vstatBASIC)) continue ;
       abarik = consys_dotcol(dy_sys,k,betai) ;
+      abari[k] = abarik ;
+      if (flgon(statk,vstatNBFX)) continue ;
       if (withintol(abarik,0,dy_tols->zero)) continue ;
-      if ((flgon(statk,vstatNBLB|vstatNBFR) && abarik < 0) ||
-	  (flgon(statk,vstatNBUB|vstatNBFR) && abarik > 0))
+      if (flgon(statk,vstatSB|vstatNBFR) ||
+	  (flgon(statk,vstatNBLB) && abarik < 0) ||
+	  (flgon(statk,vstatNBUB) && abarik > 0))
       { rayUp = FALSE ;
-	break ; }
-      abari[k] = abarik ; }
+	break ; } }
 #   ifndef DYLP_NDEBUG
     if (dy_opts->print.rays >= 4)
     { if (rayUp == FALSE)
-      { k-- ;
-	dyio_outfmt(dy_logchn,dy_gtxecho,
+      { dyio_outfmt(dy_logchn,dy_gtxecho,
 		    "; %s %s (%d): abar<%d,%d> = %g; %s (%d) ",
 		    dy_prtvstat(statk),consys_nme(dy_sys,'v',k,FALSE,NULL),k,
 		    i,k,abarik) ;
@@ -762,20 +917,22 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
 */
   if (rayDown == TRUE)
   { for (k = 1 ; k <= n ; k++)
-    { statk = getflg(dy_status[k],vstatSTATUS) ;
-      if (flgon(statk,vstatBASIC|vstatNBFX)) continue ;
+    { abari[k] = 0 ;
+      statk = getflg(dy_status[k],vstatSTATUS) ;
+      if (flgon(statk,vstatBASIC)) continue ;
       abarik = consys_dotcol(dy_sys,k,betai) ;
+      abari[k] = abarik ;
+      if (flgon(statk,vstatNBFX)) continue ;
       if (withintol(abarik,0,dy_tols->zero)) continue ;
-      if ((flgon(statk,vstatNBLB|vstatNBFR) && abarik > 0) ||
-	  (flgon(statk,vstatNBUB|vstatNBFR) && abarik < 0))
+      if (flgon(statk,vstatSB|vstatNBFR) ||
+	  (flgon(statk,vstatNBLB) && abarik > 0) ||
+	  (flgon(statk,vstatNBUB) && abarik < 0))
       { rayDown = FALSE ;
-	break ; }
-      abari[k] = abarik ; }
+	break ; } }
 #   ifndef DYLP_NDEBUG
     if (dy_opts->print.rays >= 4)
     { if (rayDown == FALSE)
-      { k-- ;
-	dyio_outfmt(dy_logchn,dy_gtxecho,
+      { dyio_outfmt(dy_logchn,dy_gtxecho,
 		    "; %s %s (%d): abar<%d,%d> = %g; %s (%d) ",
 		    dy_prtvstat(statk),consys_nme(dy_sys,'v',k,FALSE,NULL),k,
 		    i,k,abarik) ;
@@ -794,14 +951,14 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
   { dyio_outfmt(dy_logchn,dy_gtxecho,
 		"\n    active ray %s (%d)\n      non-zeros:",
 		consys_nme(dy_sys,'c',i,FALSE,NULL),i) ;
-    k = 0 ;
-    for (j = 1 ; j <= n ; j++)
-    { abarik = abari[j] ;
+    j = 0 ;
+    for (k = 1 ; k <= n ; k++)
+    { abarik = abari[k] ;
       if (withintol(abarik,0,dy_tols->zero)) continue ;
       dyio_outfmt(dy_logchn,dy_gtxecho," (%s (%d) %g)",
-		  consys_nme(dy_sys,'v',j,FALSE,NULL),j,abarik) ;
-      k++ ;
-      if (k%3 == 0) dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t\t") ; } }
+		  consys_nme(dy_sys,'v',k,FALSE,NULL),k,abarik) ;
+      j++ ;
+      if (j%3 == 0) dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t\t") ; } }
 # endif
   
 /*
@@ -830,7 +987,8 @@ static void testForDualRay (int i, int *p_dir, double **p_abari)
   return ; }
 
 
-bool dy_dualRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
+bool dy_dualRays (lpprob_struct *orig_lp, bool fullRay,
+		  int *p_numRays, double ***p_rays, bool trueDuals)
 
 /*
   This routine returns the dual rays emanating from the current basic
@@ -841,17 +999,31 @@ bool dy_dualRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
   considered an error (in judgment, at the least). A call when the result of
   optimisation was anything other than infeasible will return zero rays.
 
-  The ray returned is an m-vector in the original system frame of reference,
-  with the coefficients listed in row order. We're keeping our head firmly in
-  the sand with respect to the duals that would be associated with the upper
-  and lower bounds on variables, were those constraints to be made explicit.
-  Nor are we interested in basic dual logicals.
+  The ray returned depends on the value of fullRay.
+
+    * If fullRay is true, the full ray is returned as an (m+n)-vector in the
+      original system frame of reference. The first m entries hold ray
+      components for the duals associated with explicit constraints, listed
+      in row order. The remaining n entries hold ray components for the duals
+      that would be associated with tight bound constraints (if those
+      constraints were explicit), listed in column order.
+
+    * If fullRay is false, duals associated with tight bound constraints are
+      ignored. The ray components for the duals associated with the explicit
+      constraints are returned as above.
+
+  We're not interested in dual logicals.
+
+  The meaning of trueDuals is explained in full in the preamble to
+  dy_solutions.c.
 
   (*) Not to belabour the point, but it's possible for a problem to be dual
       and primal infeasible. In this case, you'll get no rays.
 
   Parameters:
     orig_lp:	the lp problem structure
+    fullRay:	TRUE to return the full ray, FALSE to return only the ray
+		components associated with explicit constraints.
     p_numRays:	(i) the maximum number of rays to return
 		(o) the actual number of rays returned
     p_rays:	(i) vector of (double *) or NULL; 0-based indexing
@@ -862,21 +1034,24 @@ bool dy_dualRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
 		    returned
 		(o) p_numRays entries will point to rays; each ray is an
 		    m-vector in original system row order.
+    trueDuals	if TRUE, return the ray with a sign convention appropriate
+		for the true dual problem; if false, use a sign convention
+		appropriate for the given min primal with implicit bounds.
 
   Returns: TRUE if no errors occurred while searching for rays; FALSE
 	   otherwise.
 */
 
-{ int m,n,i,m_orig,n_orig,i_orig,j_orig ;
+{ int m,n,i,j,m_orig,n_orig,i_orig,j_orig,rayLen ;
   bool error ;
   double *sc_abari ;
 
   consys_struct *orig_sys ;
   bool scaled ;
   const double *rscale,*cscale ;
-  double Si,dir ;
+  double Si,sc_abarij,rayk ;
 
-  int numRows, numRays,maxRays,rayDir,i_ray,i_orig_ray,bv_ray,bv_orig_ray ;
+  int numRays,maxRays,rayDir,i_ray,i_orig_ray,bv_ray,bv_orig_ray ;
   flags statbv_ray ;
   double **rayCollection ;
   bool ourCollection,logical ;
@@ -940,91 +1115,196 @@ bool dy_dualRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
   Set up to walk the rows of the active system.  For each row that tests out
   as a ray, testForDualRay will return abar<i> in column order in the active
   reference frame. We'll unscale it and simultaneously drop the coefficients
-  into a vector in row order in the original reference frame.
+  into the proper place in the ray, in the original reference frame.
 */
   n_orig = orig_sys->varcnt ;
   m_orig = orig_sys->concnt ;
+  if (fullRay == TRUE)
+  { rayLen = m_orig+n_orig ; }
+  else
+  { rayLen = m_orig ; }
   n = dy_sys->varcnt ;
   m = dy_sys->concnt ;
   error = FALSE ;
   numRays = 0 ;
   rayDir = 0 ;
 /*
-  Start to walk the rows of the active system. Unless the row would be a
-  candidate in a dual pivot, there's clearly no ray. Test for BLLB or BUUB
-  status for the associated basic variable.
+  Start to walk the rows of the active system. If the row is not a candidate
+  for a dual pivot, there can be no ray. Test for BLLB or BUUB status for the
+  associated basic variable.
 */
-  for (numRows = 1, i_ray = 1 ; numRows <= m ; numRows++, i_ray = (i_ray%m)+1)
+  for (i_ray = 1 ; i_ray <= m ; i_ray++)
   { bv_ray = dy_basis[i_ray] ;
     statbv_ray = dy_status[bv_ray] ;
     if (!flgon(statbv_ray,vstatBLLB|vstatBUUB)) continue ;
 /*
-  The row is not obviously unqualified, so call for a thorough check.
+  The row is not obviously unqualified, so call for a thorough check.  If we
+  have a ray, allocate a vector to hold it.
 */
     testForDualRay(i_ray,&rayDir,&sc_abari) ;
     if (rayDir == 0) continue ;
-/*
-  We have a ray. We need to unscale it and translate it from active system
-  column order to original system row order. Begin by allocating a vector
-  to hold the ray.
-
-  In terms of scaling, we have sc_abar<i> = e<i>(inv(S)inv(B)NS<j>) and we need
-  to remove the leading and trailing column scaling, if present.
-
-  Getting the ray pointed in the right direction takes some work. Playing
-  fast and loose with notation, we're testing y = cbar - abar<i>y<i>, so
-  ray<i> = -abar<i>. Then, if y<i> is decreasing, that's another factor of -1
-  (encoded in rayDir). Finally, if the constraint in question is a >=
-  constraint in the original system, there's another factor of -1 folded into
-  the row scaling (but we can't use that directly; we want only the sign).
-
-  In terms of change of reference frame, we're moving from active system
-  column order to original system row order. But since we're only interested
-  in the values associated with nonbasic logicals, in practice we scan only
-  the first m positions of abar<i> and the translation is simply active row
-  to original row, i -> i_orig. As mentioned above, we're keeping our head
-  firmly in the sand when it comes to bounded architecturals.
-
-  Last, but not least, y<i> is itself moving and, if it's associated with an
-  architectural constraint, must be part of the ray. Add a coefficient of
-  -1.0, adjusted for direction, directly in the original reference frame.
-*/
-    ray = CALLOC((m_orig+1),sizeof(double)) ;
+    ray = CALLOC((rayLen+1),sizeof(double)) ;
     rayCollection[numRays] = ray ;
     numRays++ ;
-    dir = -1.0*rayDir ;
-    i_orig_ray = dy_actcons[i_ray] ;
-    if (orig_sys->ctyp[i_orig_ray] == contypGE)
-    { dir = -dir ; }
+/*
+  We need to unscale the ray and translate it from active system column order
+  to original system row/column order.
 
+  Getting the ray components pointed in the right direction takes a little
+  work.  If y<i> is (apparently) decreasing because the basic primal is
+  leaving at its upper bound, that's a factor of -1 (encoded in rayDir).
+
+  Then, if the ray derives from a >= constraint in the original system,
+  there's another factor of -1 to be applied, already encoded in the row
+  scaling (rscale). BUT ... we have to be careful about semantics when the
+  basic variable is a slack.  If a >= constraint has been scaled to a <=
+  constraint, dylp has added a slack after the multiplication by -1, and that
+  slack is now BLLB on 0 <= slack <= infty. The coefficient of +1 is embedded
+  in the basis inverse; we can't change it. When we convert back to a >=
+  constraint, multiplying by -1, that +1 doesn't change! To get this right,
+  we have to interpret the logical as a surplus that's BUUB on bounds of
+  -infty <= surplus <= 0.  Check for this case and reverse the ray direction
+  (rayDir).
+
+  TO THINK ABOUT: What about range constraints? And should we be checking for
+  the scaling factor associated with this constraint (i_orig_ray) instead of
+  the basic logical (bv_orig_ray)? This could be an unnatural logical, after
+  all. I need to come up with an example that's dual unbounded and has an
+  unnatural logical basic on a >= constraint.
+
+*/
+    i_orig_ray = dy_actcons[i_ray] ;
     if (bv_ray <= m)
     { logical = TRUE ;
-      bv_orig_ray = dy_actcons[bv_ray] ; }
+      bv_orig_ray = dy_actcons[bv_ray] ;
+      if (scaled == TRUE && rscale[bv_orig_ray] < 0)
+	rayDir = -rayDir ; }
     else
     { logical = FALSE ;
       bv_orig_ray = dy_actvars[bv_ray] ; }
+/*
+  In terms of scaling, we have sc_abar<i> = e<i>(inv(S<B>)inv(B)NS<N>) and we
+  need to remove the leading and trailing column scaling, if present.
 
+  For the duals associated with implicit upper and lower bounds, it boils
+  down to `if these constraints were explicitly added to the system, we'd
+  write them as honest <= constraints, and they would have a positive dual.'
+  A rigourous explanation of how we come to have apparently negative duals
+  requires detailed examination of just how it is that you can run dual
+  simplex at all using the primal matrices and data structures. See the
+  typeset documentation on the relationship between primal and dual simplex.
+
+  NOTE: The code assumes that the lower bound constraint will take the form
+	-x<j> <= -lb. The rationale is that the canonical primal problem is
+	Ax <= b, hence it makes sense to assume this form when reconstructing
+	an implicit constraint. But this is a choice, not a mathematical
+	requirement. We could pretend these are >= constraints, at the cost
+	of some mental gymnastics.
+
+  NBFX is not an issue, as the dual for an equality is a free variable.
+
+  In terms of change of reference frame, we're moving from active system
+  column order to original system row/column order. For logicals, it's simply
+  row i -> i_orig; for columns, j -> j_orig. If the user is asking for the
+  full ray, we need to scan all of abar<i>; otherwise, only the first m
+  positions (the positions occupied by logicals).
+*/
     if (scaled == TRUE)
     { if (logical == TRUE)
-      { Si = 1/rscale[bv_orig_ray]*dir ; }
+      { Si = 1/rscale[bv_orig_ray]*rayDir ; }
       else
-      { Si = cscale[bv_orig_ray]*dir ; }
+      { Si = cscale[bv_orig_ray]*rayDir ; }
       for (i = 1 ; i <= m ; i++)
       { if (sc_abari[i] == 0) continue ;
         i_orig = dy_actcons[i] ;
-	ray[i_orig] = rscale[i_orig]*sc_abari[i]*Si ;
-	setcleanzero(ray[i_orig],dy_tols->zero) ; } }
+	rayk = Si*sc_abari[i]*rscale[i_orig] ;
+	setcleanzero(rayk,dy_tols->zero) ;
+	ray[i_orig] = rayk ; }
+      if (fullRay == TRUE)
+      { for (j = m+1 ; j <= n ; j++)
+	{ 
+#	  if DYLP_PARANOIA > 0
+	  if (dy_chkstatus(j) == FALSE)
+	  { error = TRUE ; }
+#	  endif
+	  if (sc_abari[j] == 0) continue ;
+/*
+  Deal with a ray coefficient that's negated because the corresponding
+  primal is NBUB.
+*/
+	  j_orig = dy_actvars[j] ;
+	  rayk = Si*sc_abari[j]*(1/cscale[j_orig]) ;
+	  if (trueDuals == TRUE && flgon(dy_status[j],vstatNBUB))
+	  {
+#	    if DYLP_PARANOIA > 0
+	    if (rayk > dy_tols->cost)
+	    { errmsg(739,rtnnme,dy_sys->nme,"active",
+		     consys_nme(dy_sys,'v',j,FALSE,NULL),j,
+		     dy_prtvstat(dy_status[j]),rayk) ;
+	      error = TRUE ; }
+#	    endif
+	    rayk = -rayk ; }
+	  setcleanzero(rayk,dy_tols->zero) ;
+	  ray[m_orig+j_orig] = rayk ; } } }
+/*
+  The same, but without scaling. We still need to cope with rayDir and NBUB
+  variables.
+*/
     else
     { for (i = 1 ; i <= m ; i++)
       { if (sc_abari[i] == 0) continue ;
 	i_orig = dy_actcons[i] ;
-	ray[i_orig] = sc_abari[i]*dir ;
-	setcleanzero(ray[i_orig],dy_tols->zero) ; } }
+	rayk = sc_abari[i]*rayDir ;
+	setcleanzero(rayk,dy_tols->zero) ;
+	ray[i_orig] = rayk ; }
+      if (fullRay == TRUE)
+      { for (j = m+1 ; j <= n ; j++)
+	{ 
+#	  if DYLP_PARANOIA > 0
+	  if (dy_chkstatus(j) == FALSE)
+	  { error = TRUE ; }
+#	  endif
+	  if (sc_abari[j] == 0) continue ;
+	  rayk = sc_abari[j]*rayDir ;
+	  if (trueDuals == TRUE && flgon(dy_status[j],vstatNBUB))
+	  {
+#	    if DYLP_PARANOIA > 0
+	    if (rayk > dy_tols->cost)
+	    { errmsg(739,rtnnme,dy_sys->nme,"active",
+		     consys_nme(dy_sys,'v',j,FALSE,NULL),j,
+		     dy_prtvstat(dy_status[j]),rayk) ;
+	      error = TRUE ; }
+#	    endif
+	    rayk = -rayk ; }
+	  j_orig = dy_actvars[j] ;
+	  setcleanzero(rayk,dy_tols->zero) ;
+	  ray[m_orig+j_orig] = rayk ; } } }
     if (sc_abari != NULL)
     { FREE(sc_abari) ;
       sc_abari = NULL ; }
+
+#   if DYLP_PARANOIA > 0
+/*
+  Do the paranoid check before adding the coeff for the nonbasic dual. All
+  we're testing here is e<i>inv(B)N.
+*/
+    check_dualRay(orig_lp,fullRay,i_orig_ray,rayDir,ray,trueDuals) ;
+#   endif
+
+/*
+  Last, but not least, y<i> is itself moving and must be part of the ray.
+  Add a coefficient of 1.0 in the original reference frame.  Except for a >=
+  constraint, where we need -1 because any inversion encoded in the row
+  scaling hasn't yet been applied to this coefficient.
+*/
     if (logical == TRUE)
-    { ray[bv_orig_ray] = -1.0*dir ; }
+    { if (orig_sys->ctyp[bv_orig_ray] == contypGE)
+      { ray [bv_orig_ray] = -1.0 ; }
+      else
+      { ray[bv_orig_ray] = 1.0 ; } }
+    else
+    if (fullRay == TRUE)
+    { ray[m_orig+bv_orig_ray] = 1.0 ; }
 
 #   ifndef DYLP_NDEBUG
     if (dy_opts->print.rays >= 5)
@@ -1034,18 +1314,29 @@ bool dy_dualRays (lpprob_struct *orig_lp, int *p_numRays, double ***p_rays)
       j_orig = 0 ;
       for (i_orig = 1 ; i_orig <= m_orig ; i_orig++)
       { if (ray[i_orig] != 0)
-	{ dyio_outfmt(dy_logchn,dy_gtxecho," (%s (%d) %g)",
+	{ if (j_orig != 0 && j_orig%3 == 0)
+	    dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t\t") ;
+	  dyio_outfmt(dy_logchn,dy_gtxecho," (%s (%d) %g)",
 		      consys_nme(orig_sys,'c',i_orig,FALSE,NULL),i_orig,
-		      ray[i_orig]) ; }
-	j_orig++ ;
-	if (j_orig%3 == 0) dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t\t") ; } }
+		      ray[i_orig]) ;
+	  j_orig++ ; } }
+      if (fullRay == TRUE)
+      { i_orig = j_orig ;
+	for (j_orig = 1 ; j_orig <= n_orig ; j_orig++)
+	{ if (ray[m_orig+j_orig] != 0)
+	  { if (i_orig%3 == 0)
+	      dyio_outfmt(dy_logchn,dy_gtxecho,"\n*\t\t") ;
+	    dyio_outfmt(dy_logchn,dy_gtxecho," (%s (%d) %g)",
+			consys_nme(orig_sys,'v',j_orig,FALSE,NULL),j_orig,
+			ray[m_orig+j_orig]) ;
+	    i_orig++ ; } } } }
 #   endif
 
     if (numRays >= maxRays) break ; }
 /*
   End of scanning loop. We've either found the requested number of rays,
-  scanned all columns, or encountered an error. Time for cleanup. Free the
-  vector used for abar<j>. If we have an error, free the ray collection.
+  scanned all rows, or encountered an error. Time for cleanup. Free the
+  vector used for sc_abar<i>. If we have an error, free the ray collection.
 */
   if (sc_abari != NULL) FREE(sc_abari) ;
   if (error == TRUE)
