@@ -13,8 +13,11 @@
 
 /*
   This file contains routines which return primal and dual solutions, unscaled,
-  in the original system frame of reference. For primal variables, there are
-  three routines:
+  in the original system frame of reference.
+  
+  PRIMAL VARIABLES
+  
+  For primal variables, there are three routines:
 
     * dy_rowPrimals:	the primal basic variables x<B> = inv(B)b, in basis
 			(row) order
@@ -35,7 +38,10 @@
     * dy_logStatus:	the status of the primal logical variables, in row
 			order
 
-  For dual variables, there are two routines:
+  DUAL VARIABLES
+
+  Dual variables are complicated because we're not really running dual simplex,
+  we're faking it on the primal constraint system. There are two routines:
 
     * dy_rowDuals:	the dual variables y = c<B>inv(B) associated with the
 			architectural constraints, in basis (row) order
@@ -50,21 +56,45 @@
   nonbasic primal architectural variables and are a mixture of architectural
   and logical duals.
 
-  There is another utility, dy_orig_soln, which is (at present a fairly
-  specialised routine used by dylp_utils:buildsoln. It should be rewritten to
-  conform to the pattern of the previous four.
+  There's one more thing to be considered with respect to dual variables: we
+  have to choose a sign convention.  Because we're faking dual simplex on the
+  primal data structures, we have a consistency problem.  With the usual max
+  primal <=> min dual pair, the row duals y = c<B>inv(B) are the correct
+  sign, and the reduced costs (when interpreted as dual variables) are
+  negated.  Because dylp works with a min primal <=> min dual pair, the row
+  duals are negated and the reduced costs are correct.  One set of values
+  must be negated for consistency, but which one?  Just to complicate the
+  issue a bit further, the canonical primal <=> dual pair assumes that the
+  primal is Ax <= b. Dylp automagically converts >= constraints to <=
+  constraints by hiding the -1 in the row scaling factor.  The client likely
+  expects that the duals returned to him/her will work with the original mix
+  of >= and <= constraints.
 
-  There's a routine, dy_expandxopt, which takes the primal solution generated
-  by dy_orig_soln (which contains basic variables only) and expands it to a
-  full solution (all architectural variables). This, too, should be pulled from
-  dylp_utils and moved here (or perhaps made obsolete).
+  Dylp sidesteps this issue by adding a boolean to dy_rowDuals and
+  dy_colDuals (and also to dy_rays.c:dy_dualRays).  A value of false gives
+  you duals with a sign convention appropriate for a min primal problem.  A
+  value of true gives you the true duals, i.e., the duals that would result
+  from the standard max primal <=> min dual pairing. In either case, the sign
+  of the dual is flipped for a >= constraint in the original system.
+
+  MISCELLANEOUS
+
+  These routines predate the comprehensive set of solution routines described
+  above.
+
+  Dy_orig_soln is a utility which is (at present) a fairly specialised
+  routine used by dylp_utils:buildsoln.
+
+  There's also a routine, dy_expandxopt, which takes the primal solution
+  generated by dy_orig_soln (which contains basic variables only) and expands
+  it to a full solution (all architectural variables).
 */
 
 #define DYLP_INTERNAL
 
 #include "dylp.h"
 
-static char svnid[] UNUSED = "$Id: dy_solutions.c 269 2009-04-02 05:38:19Z lou $" ;
+static char svnid[] UNUSED = "$Id: dy_solutions.c 301 2009-10-05 20:48:11Z lou $" ;
 
 #ifdef DYLP_PARANOIA
 extern bool dy_std_paranoia (const lpprob_struct *orig_lp,
@@ -73,7 +103,7 @@ extern bool dy_std_paranoia (const lpprob_struct *orig_lp,
 
 
 
-void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
+void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar, bool trueDuals)
 
 /*
   Returns the unscaled vector of duals associated with architectural columns
@@ -81,7 +111,14 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
 
   These are the duals associated with implicit bound constraints. See
   dy_rowDuals for the duals associated with explicit (architectural)
-  constraints. (These latter are the usual notion of dual variables.)
+  constraints. (These latter are the usual notion of dual variables, and
+  also correspond to the reduced costs of logical variables.)
+
+  In dylp's min primal <=> min dual pairing, the reduced costs have the
+  correct sign for the true dual variables used by the min dual problem,
+  except that the values associated with NBUB variables need to be negated.
+  If you'd prefer that the duals have a sign convention appropriate for a min
+  primal, specify trueDuals = false.
 
   The algorithm is to walk the columns of orig_sys, copying over the reduced
   cost from dy_cbar when the variable is active, otherwise calculting cbar<j>
@@ -112,6 +149,9 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
     p_cbar:	(i) pointer to vector; if NULL, a vector of the appropriate
 		    size will be allocated
 		(o) vector of reduced costs
+    trueDuals:	true to return values with a sign convention appropriate
+		for the min dual problem, false to use a sign convention that
+		matches the min primal.
 
   Returns: undefined
 */
@@ -123,10 +163,10 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
   double *orig_y ;
   consys_struct *scaled_orig_sys ;
   bool scaled ;
+  const double *rscale,*cscale ;
 
   double cbarj ;
   double *cbar ;
-  const double *rscale,*cscale ;
 
 # ifdef DYLP_PARANOIA
   char *rtnnme = "dy_colDuals" ;
@@ -139,7 +179,8 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
 # endif
 /*
   Is unscaling required? Acquire the scaling vectors and set up scaled_orig_sys
-  accordingly.
+  accordingly. We'll also need the constraint type vector so that we don't
+  overcompensate for >= constraints when returning true duals.
 */
   scaled = dy_isscaled() ;
   if (scaled == TRUE)
@@ -160,7 +201,7 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
   { cbar = *p_cbar ;
     memset(cbar,0,(n_orig+1)*sizeof(double)) ; }
   else
-  { cbar = (double *) CALLOC((n+1),sizeof(double)) ; }
+  { cbar = (double *) CALLOC((n_orig+1),sizeof(double)) ; }
 /*
   Make a vector of duals that matches orig_sys, for efficient pricing of
   inactive columns.
@@ -183,9 +224,13 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
       if (flgon(statj,vstatBASIC))
       { cbarj = 0.0 ; }
       else
-      { cbarj = dy_cbar[j]/cscale[j_orig] ; } }
+      { if (scaled == TRUE)
+	{ cbarj = dy_cbar[j]/cscale[j_orig] ; }
+	else
+	{ cbarj = dy_cbar[j] ; } } }
     else
-    { if (scaled == TRUE)
+    { statj = (flags) -dy_origvars[j_orig] ;
+      if (scaled == TRUE)
       { cbarj = scaled_orig_sys->obj[j_orig] ; 
 	cbarj -= consys_dotcol(scaled_orig_sys,j_orig,orig_y) ;
 	cbarj /= cscale[j_orig] ; }
@@ -193,7 +238,18 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
       { cbarj = orig_sys->obj[j_orig] ;
 	cbarj -= consys_dotcol(orig_sys,j_orig,orig_y) ; } }
     setcleanzero(cbarj,dy_tols->cost) ;
-    cbar[j_orig] = cbarj ; }
+/*
+  What's our sign convention? If these values are to work with the imaginary
+  true dual problem, we need to flip the sign on variables that are NBUB. If
+  we're just going for the min primal convention, they're already correct.
+*/
+    if (trueDuals == TRUE)
+    { if (flgon(statj,vstatNBUB))
+	cbar[j_orig] = -cbarj ;
+      else
+	cbar[j_orig] = cbarj ; }
+    else
+      cbar[j_orig] = cbarj ; }
 /*
   Clean up a bit and we're done.
 */
@@ -204,17 +260,24 @@ void dy_colDuals (lpprob_struct *orig_lp, double **p_cbar)
 
 
 
-void dy_rowDuals (lpprob_struct *orig_lp, double **p_y)
+void dy_rowDuals (lpprob_struct *orig_lp, double **p_y, bool trueDuals)
 
 /*
   This routine returns the unscaled vector of row duals, commonly referred to
   as the dual variables, c<B>inv(B). The values are unscaled and returned in a
   vector matching the original system frame of reference. Duals associated with
-  inactive rows are always zero. The relevant bit of unscaling is:
+  inactive rows are always zero.
+  
+  In dylp's min primal <=> min dual pairing, the duals have the wrong sign
+  for the true dual variables used by the min dual problem. If you'd prefer
+  that the duals have a sign convention appropriate for the min dual problem,
+  specify trueDuals = false.
 
-  sc_y<i> = - sc_c<B>sc_inv(B)
-	  = - c<B>S<B>inv(S<B>)inv(B)inv(R)
-	  = - c<B>inv(B)inv(R)
+  The relevant bit of unscaling is:
+
+  sc_y<i> = sc_c<B>sc_inv(B)
+	  = c<B>S<B>inv(S<B>)inv(B)inv(R)
+	  = c<B>inv(B)inv(R)
 
   So, to recover y<i> we need to postmultiply by inv(R). The appropriate row
   factor is the one associated with the original row.
@@ -234,6 +297,7 @@ void dy_rowDuals (lpprob_struct *orig_lp, double **p_y)
   double *y ;
 
   consys_struct *orig_sys ;
+  contyp_enum *ctyp ;
 
   bool scaled ;
   const double *rscale,*cscale ;
@@ -264,6 +328,7 @@ void dy_rowDuals (lpprob_struct *orig_lp, double **p_y)
   m_orig = orig_sys->concnt ;
   n = dy_sys->varcnt ;
   m = dy_sys->concnt ;
+  ctyp = orig_sys->ctyp ;
 /*
   Do we need a vector?
 */
@@ -285,15 +350,21 @@ void dy_rowDuals (lpprob_struct *orig_lp, double **p_y)
       setcleanzero(yi,dy_tols->cost) ; }
     else
     { yi = 0.0 ; }
-    y[i_orig] = yi ; }
+/*
+  The true duals are the negative of the minimisation duals here.
+*/
+    if (trueDuals == TRUE)
+      y[i_orig] = -yi ;
+    else
+      y[i_orig] = yi ; }
 
 # ifndef DYLP_NDEBUG
   if (dy_opts->print.soln >= 3)
-  { dyio_outfmt(dy_logchn,dy_gtxecho,"\ty =") ;
+  { dyio_outfmt(dy_logchn,dy_gtxecho,"\n\ty =") ;
     v = 0 ;
     for (i_orig = 1 ; i_orig <= m_orig ; i_orig++)
     { if (y[i_orig] != 0)
-      { if ((++v)%4 == 0)
+      { if ((++v)%3 == 0)
 	{ v = 0 ;
 	  dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t   ") ; }
 	i = dy_origcons[i_orig] ;
@@ -416,11 +487,11 @@ void dy_colPrimals (lpprob_struct *orig_lp, double **p_x)
 
 # ifndef DYLP_NDEBUG
   if (dy_opts->print.soln >= 3)
-  { dyio_outfmt(dy_logchn,dy_gtxecho,"\tx =") ;
+  { dyio_outfmt(dy_logchn,dy_gtxecho,"\n\tx =") ;
     v = 0 ;
     for (j_orig = 1 ; j_orig <= n_orig ; j_orig++)
     { if (x[j_orig] != 0)
-      { if ((++v)%4 == 0)
+      { if ((++v)%3 == 0)
 	{ v = 0 ;
 	  dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t   ") ; }
 	dyio_outfmt(dy_logchn,dy_gtxecho," (%d %g %s)",
@@ -453,8 +524,11 @@ void dy_rowPrimals (lpprob_struct *orig_lp, double **p_xB, int **p_indB)
   logicals, recall that S<i> = 1/R<i>.
 
   By construction, the basic variable for inactive constraints is the logical
-  for the constraint. Obtaining beta<i> for an inactive row and calculating
-  dot(beta<i>,b) is a lot of work. Use b<i> - dot(a<i>,x) instead.
+  for the constraint. Generating beta<i> = [ -a<B,i>inv(B) 1 ] for an inactive
+  row, correcting b<i> for nonbasic, nonzero variables (active and inactive),
+  and calculating dot(beta<i>,b) is a lot of work. Much easier to call
+  colPrimals for the complete solution vector and calculate b<i> - dot(a<i>,x)
+  in the original system.
 
   Parameters:
     orig_lp:	the original lp problem
@@ -465,7 +539,9 @@ void dy_rowPrimals (lpprob_struct *orig_lp, double **p_xB, int **p_indB)
     p_indB:	(i) vector to hold the indices of the primal basic variables;
 		    if NULL, a vector of appropriate size will be allocated
 		(o) indices of the primal basic variables, unscaled, in the
-		    original system frame of reference
+		    original system frame of reference; indices of logical
+		    variables are encoded as the negative of the constraint
+		    index
 
   Returns: undefined
 */
@@ -529,9 +605,9 @@ void dy_rowPrimals (lpprob_struct *orig_lp, double **p_xB, int **p_indB)
   else
   { indB = (int *) CALLOC((m_orig+1),sizeof(int)) ; }
 /*
-  Walk the columns of the original system. For each constraint that's active,
-  we can obtain the value from dy_xbasic. For each inactive constraint, we
-  need to calculate the value of the logical.
+  Walk the constraints of the original system. For each constraint that's
+  active, we can obtain the value from dy_xbasic. For each inactive constraint,
+  we need to calculate the value of the logical.
 
   Indices of logicals are recorded in indB as the negative of the constraint
   index.
@@ -567,10 +643,10 @@ void dy_rowPrimals (lpprob_struct *orig_lp, double **p_xB, int **p_indB)
 
 # ifndef DYLP_NDEBUG
   if (dy_opts->print.soln >= 3)
-  { dyio_outfmt(dy_logchn,dy_gtxecho,"\txB =") ;
+  { dyio_outfmt(dy_logchn,dy_gtxecho,"\n\txB =") ;
     v = 0 ;
     for (i_orig = 1 ; i_orig <= m_orig ; i_orig++)
-    { if ((++v)%4 == 0)
+    { if ((++v)%3 == 0)
       { v = 0 ;
 	dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t   ") ; }
       j_orig = indB[i_orig] ;
@@ -691,12 +767,12 @@ void dy_logPrimals (lpprob_struct *orig_lp, double **p_logx)
 
 # ifndef DYLP_NDEBUG
   if (dy_opts->print.soln >= 3)
-  { dyio_outfmt(dy_logchn,dy_gtxecho,"\tlogx =") ;
+  { dyio_outfmt(dy_logchn,dy_gtxecho,"\n\tlogx =") ;
     n_orig = orig_sys->varcnt ;
     v = 0 ;
     for (i_orig = 1 ; i_orig <= m_orig ; i_orig++)
     { if (logx[i_orig] != 0)
-      { if ((++v)%4 == 0)
+      { if ((++v)%3 == 0)
 	{ v = 0 ;
 	  dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t   ") ; }
 	dyio_outfmt(dy_logchn,dy_gtxecho," (%d %g %s)",
@@ -775,10 +851,10 @@ void dy_colStatus (lpprob_struct *orig_lp, flags **p_colstat)
 
 # ifndef DYLP_NDEBUG
   if (dy_opts->print.soln >= 3)
-  { dyio_outfmt(dy_logchn,dy_gtxecho,"\tcolstat =") ;
+  { dyio_outfmt(dy_logchn,dy_gtxecho,"\n\tcolstat =") ;
     v = 0 ;
     for (j_orig = 1 ; j_orig <= n_orig ; j_orig++)
-    { if ((++v)%4 == 0)
+    { if ((++v)%3 == 0)
       { v = 0 ;
 	dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t   ") ; }
       dyio_outfmt(dy_logchn,dy_gtxecho," (%s %d %s)",
@@ -806,7 +882,7 @@ void dy_logStatus (lpprob_struct *orig_lp, flags **p_logstat)
   It's actually a fair bit of work to get the status right for inactive
   constraints. Because we're reporting the full set of dylp status codes, and
   the client might be calling in a situation where the outcome was infeasible
-  or unbounded, wee need to calculate the value and assign the appropriate
+  or unbounded, we need to calculate the value and assign the appropriate
   status code. 
 
   Parameters:
@@ -924,11 +1000,11 @@ void dy_logStatus (lpprob_struct *orig_lp, flags **p_logstat)
 
 # ifndef DYLP_NDEBUG
   if (dy_opts->print.soln >= 3)
-  { dyio_outfmt(dy_logchn,dy_gtxecho,"\trowstat =") ;
+  { dyio_outfmt(dy_logchn,dy_gtxecho,"\n\trowstat =") ;
     n_orig = orig_sys->varcnt ;
     v = 0 ;
     for (i_orig = 1 ; i_orig <= m_orig ; i_orig++)
-    { if ((++v)%4 == 0)
+    { if ((++v)%3 == 0)
       { v = 0 ;
 	dyio_outfmt(dy_logchn,dy_gtxecho,"\n\t   ") ; }
       dyio_outfmt(dy_logchn,dy_gtxecho," (%s %d %s)",
